/*  author: G. Matas                           (g.matas@ee.surrey.ac.uk)
*/
/* +-------------------------------------------------------------------+ */
/* | Copyright 1992, George Matas.                                     | */
/* |   Permission to use, copy, modify, and distribute this software   | */
/* |   and its documentation for any purpose and without fee is hereby | */
/* |   granted, provided that the above copyright notice appear in all | */
/* |   copies and that both that copyright notice and this permission  | */
/* |   notice appear in supporting documentation.  This software is    | */
/* |   provided "as is" without express or implied warranty.           | */
/* +-------------------------------------------------------------------+
*/
/*-----------------------------------------------------------------------*/
static char sccsid[]="@(#)93/06/30 g.matas@ee.surrey.ac.uk 5.1 sort_para.c";



#include "LL.h"

#include <stdio.h>                                   /* for printf, gets etc */
#include <string.h>                                        /* for strcmp etc */


/*------------------ LL library example ---------------------------------
    1.   Reads in a text file from the stdio  ( sort_ex  <file)
    2.   Creates a list of paragraphs . A paragraph and at a blank line.
	   Sort lines inside a paragraph by alphabetical order.
    3.   Sorts the list of paragraphs according to the number of lines
    4.   prints out the whole file 

      note:  to get the last paragraph, it must be terminated by a blank line

  DEMONSTRATES:
    a/  how to create a list of list
    b/  how simple it is to sort a file using the library
    c/  how to store a string (not a pointer to it) in a list
-----------------------------------------------------------------------*/
/* declarations missing in our standard headers ...  */

static int LengthCmp(const void * par1, const void * par2)
{  
   int l1 = SizeLL(*(t_LL const *)par1);
   int l2 = SizeLL(*(t_LL const *)par2);
   if (l1==l2) return 0;
   return (l1<l2)? -1 : 1 ;
}
static int sCmp(const void * s1, const void * s2) { return strcmp(s1,s2);}

int main(void)
{
  char  buffer[200];
  int   line_length;
  t_LL all_paragraphs = ConsLL();
  t_LL paragraph     = ConsLL();

  while(-1!=scanf("%[^\n]",buffer)){
    line_length = strlen(buffer);
    if (0 == line_length) {                   /* end of paragraph  */
      SortLL(paragraph, sCmp);                /* alphabetical sort */
      InsLastLL(all_paragraphs, paragraph);  
      paragraph = ConsLL();
      scanf("%*c");                           /* skip the empty line */
    }
    else{
      InsLastLLf(paragraph,strlen(buffer)+1,buffer);  
      scanf("%*c");                            /* skip the empty line */
    }
    buffer[0]='\0';
  }
   

	
  SortLL(all_paragraphs, LengthCmp);   /* sort paragraphs according to length*/

  {
    char * line;
    t_LL * p_paragraph;
    int i =0;

    ForeachLL_M(all_paragraphs,p_paragraph){
      printf("para no %i: \n",i++);
      ForeachLL_M(*p_paragraph, line)
	printf("%s\n",line);
      
      printf("\n\n");
    }
  }
  
  {                                 /* Destroy all sub-lists (paragraphs) */
    t_LL * p_paragraph;
    ForeachLL_M(all_paragraphs,p_paragraph)
      DestLL(*p_paragraph);
  }
  
  DestLL(all_paragraphs);

  return 0;
}


